from qgis.PyQt.QtWidgets import QDialog
from qgis.PyQt.QtCore import QVariant
from qgis.core import QgsVectorLayer, QgsFeature, QgsPointXY, QgsGeometry, QgsProject, QgsField

from .panels import Panels
import requests
import re

class IsochronePanel():
    def __init__(self, dialog: QDialog, plugin) -> None:
        self._plugin = plugin
        self._dialog = dialog

    def setup_panel(self) -> None:
        print("Setup Isochrone panel")

        self.display_error()
        self._dialog.isochrone_create_push_button.clicked.connect(self.clicked_btn)

        # TODO: Get list with point layers
        # point_layer_list = []
        # for layer in QgsProject.instance().mapLayers().values():
        #     if type(layer) is QgsVectorLayer:
        #         if layer.geometryType() == QgsWkbTypes.PointGeometry:
        #             point_layer_list.append(layer.name())
        # print(point_layer_list)

    def make_active_panel(self):
        print('isochrone make active')
        self.display_error()

    def display_error(self):
        if self._plugin.error_message is None:
            self._dialog.isochrone_error_label.setVisible(False)
        else:
            self._dialog.isochrone_error_label.setText(self._plugin.error_message)
            self._dialog.isochrone_error_label.setVisible(True)


    def clicked_btn(self):

        print("clicked")


        latLineEditText = self._dialog.isochrone_lat_line_edit.text()
        lonLineEditText = self._dialog.isochrone_lon_line_edit.text()
        
        isochrone_travel_times_check_box_five_checked = self._dialog.isochrone_travel_times_check_box_five.isChecked()
        isochrone_travel_times_check_box_ten_checked = self._dialog.isochrone_travel_times_check_box_ten.isChecked()
        isochrone_travel_times_check_box_fifteen_checked = self._dialog.isochrone_travel_times_check_box_fifteen.isChecked()
        isochrone_travel_times_check_box_twenty_checked = self._dialog.isochrone_travel_times_check_box_twenty.isChecked()
        isochrone_travel_times_check_box_twentyfive_checked = self._dialog.isochrone_travel_times_check_box_twentyfive.isChecked()
        isochrone_travel_times_check_box_thirty_checked = self._dialog.isochrone_travel_times_check_box_thirty.isChecked()

        isochrone_modality_radio_button_walking_checked = self._dialog.isochrone_modality_radio_button_walking.isChecked()
        isochrone_modality_radio_button_biking_checked = self._dialog.isochrone_modality_radio_button_biking.isChecked()
        isochrone_modality_radio_button_driving_checked = self._dialog.isochrone_modality_radio_button_driving.isChecked()

        # TODO: isochroon voor ieder punt
        # lat = None
        # lon = None
        # selectedLayer = QgsProject.instance().mapLayersByName('test')
        # if not len(selectedLayer) == 1:
        #     print("Wrong lat input")
        #     self._plugin.error_message = 'Error in latitude'
        #     self.showIsochonesWindow()
        #     return

        # if type(selectedLayer[0]) is QgsVectorLayer:
        #     if selectedLayer[0].geometryType() == QgsWkbTypes.PointGeometry:
        #         for feature in selectedLayer[0].getFeatures():
        #             # TODO: check if feature is multipoint
        #             geometry = QgsGeometry.asPoint(feature.geometry())#get the geometry of the feature
        #             point = QgsPointXY(geometry)
        #             lon = point.x() # lan
        #             lat = point.y() # lat


        # Get Lat Long
        if not bool(re.match("^[0-9]{1,3}(?:[\.,][0-9]+)?$", latLineEditText)):
            print("Wrong lat input")
            self._plugin.error_message = 'Error in latitude'
            # self.showIsochonesWindow()
            self.display_error()
            return
        if not bool(re.match("^[0-9]{1,3}(?:[\.,][0-9]+)?$", lonLineEditText)):
            print("Wrong lon input")
            self._plugin.error_message = 'Error in longitude'
            # self.showIsochonesWindow()
            self.display_error()
            return

        lat = float(latLineEditText.replace(',', '.')) 
        lon = float(lonLineEditText.replace(',', '.'))

        # Travel Times
        travelTimesList = []

        if isochrone_travel_times_check_box_five_checked:
            travelTimesList.append(5)
        if isochrone_travel_times_check_box_ten_checked:
            travelTimesList.append(10)
        if isochrone_travel_times_check_box_fifteen_checked:
            travelTimesList.append(15)
        if isochrone_travel_times_check_box_twenty_checked:
            travelTimesList.append(20)
        if isochrone_travel_times_check_box_twentyfive_checked:
            travelTimesList.append(25)
        if isochrone_travel_times_check_box_thirty_checked:
            travelTimesList.append(30)

        if len(travelTimesList) == 0:
            print("At least one travel time should be checked")
            self._plugin.error_message = 'Error: travel time not selected'
            # self.showIsochonesWindow()
            self.display_error()
            return

        # Modality
        modality = ""
        if isochrone_modality_radio_button_walking_checked:
            modality = "walk"
        if isochrone_modality_radio_button_biking_checked:
            modality = "bike"
        if isochrone_modality_radio_button_driving_checked:
            modality = "car"

        if modality == "":
            print("One form of modality should be checked")
            self._plugin.error_message = 'Error: modality not selected'
            # self.showIsochonesWindow()
            self.display_error()
            return

        print(self._plugin.api_key)

        # Make request
        url = self._plugin.api_url+"/routing/v1/isochrones"
        requestBody = {
            "latitude": lat,
            "longitude": lon,
            "minutes": travelTimesList,
            "modality": modality
        }
        headers = {'Content-type': 'application/json', 'Authorization': self._plugin.api_key}
        r = requests.post(url, json=requestBody, headers=headers)

        if r.status_code == 200:
            print(r.json())
            self.drawIsochrone(r.json())
            self._plugin.error_message = None
            self.display_error()
            self._dialog.hide()
        elif r.status_code == 403 or r.status_code == 401:
            print("Error form server: ", r.status_code)
            self._plugin.error_message = 'Provide a valid API key and try again'
            # self.showConfigurationWindow()
            self._dialog.switch_panel(Panels.Settings)
        else:
            print("Error form server: ", r.status_code)
            self._plugin.error_message = 'Server error'
            # self.showIsochonesWindow()
            self.display_error()


    def drawIsochrone(self, geoJson):
        """Draw GeoJson on new layer"""

        if "features" in geoJson and "properties" in geoJson:
            
            newFeatures = []
            for jsonFeature in geoJson["features"]:
                if "geometry" in jsonFeature and "properties" in jsonFeature:
                    if "coordinates" in jsonFeature["geometry"]:

                        newCoordinates = []
                        for coordinates in jsonFeature["geometry"]["coordinates"]:
                            newCoordinatesInnerList = []
                            for coordinate in coordinates:
                                if len(coordinate) == 2:
                                    newCoordinatesInnerList.append(QgsPointXY(coordinate[0], coordinate[1]))
                            newCoordinates.append(newCoordinatesInnerList)

                        newAttributes = ["", "", 0]
                        if "modality" in jsonFeature["properties"]:
                            newAttributes[0] = jsonFeature["properties"]["modality"]
                        if "speed_type" in jsonFeature["properties"]:
                            newAttributes[1] = jsonFeature["properties"]["speed_type"]
                        if "travel_time_minute" in jsonFeature["properties"]:
                            newAttributes[2] = jsonFeature["properties"]["travel_time_minute"]

                        feature = QgsFeature()
                        feature.setGeometry(QgsGeometry.fromPolygonXY(newCoordinates))
                        feature.setAttributes(newAttributes)
                        newFeatures.append(feature)

            if len(newFeatures) > 0 and "latitude" in geoJson["properties"] and "longitude" in geoJson["properties"]:

                # point
                pointLayer =  QgsVectorLayer('Point', 'StartingPoint' , "memory")
                pointDataProvider = pointLayer.dataProvider() 

                pt = QgsFeature()
                point1 = QgsPointXY(geoJson["properties"]["longitude"], geoJson["properties"]["latitude"])
                pt.setGeometry(QgsGeometry.fromPointXY(point1))
                pointDataProvider.addFeatures([pt])

                pointLayer.updateExtents()

                # polygon
                polygonLayer = QgsVectorLayer('Polygon?crs=epsg:4326', 'Isochrones' , 'memory')
                polygonDataProvider = polygonLayer.dataProvider()

                polygonDataProvider.addAttributes([QgsField("modality", QVariant.String), QgsField("speed_type", QVariant.String), QgsField("travel_time_minute", QVariant.Int)])
                polygonLayer.updateFields()

                polygonDataProvider.addFeatures(newFeatures)
                polygonLayer.updateExtents()

                # add layers
                QgsProject.instance().addMapLayers([pointLayer, polygonLayer])


